﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Interpreter;

namespace HeuristicLab.Algorithms.PushGP.Cli
{
    class Program
    {
        static void Main(string[] args)
        {
            //Stepwise().Wait();
            PerformanceTest();
            //PerformanceTestCodeGenerator();

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();
        }

        static async Task Stepwise()
        {
            var program = PushGPInterpreter.Encode("( 2 3 INTEGER.* 4.1 5.2 FLOAT.+ TRUE FALSE BOOLEAN.OR )");
            var interpreter = new PushGPInterpreter();

            var task = interpreter.InterpreteAsync(program);
            await interpreter.PauseAsync();

            while (!interpreter.IsCompleted)
            {
                Console.Clear();
                interpreter.PrintStacks();
                interpreter.Step();

                var input = Console.ReadKey();
                if (input.Key == ConsoleKey.Escape)
                {
                    break;
                }
                else if (input.Key == ConsoleKey.Spacebar)
                {
                    await interpreter.ResumeAsync();
                }
            }

            Console.Clear();
            interpreter.PrintStacks();
        }

        static void PerformanceTest()
        {
            var program = PushGPInterpreter.Encode("( 5 INTEGER.DUP INTEGER.+ )");
            var interpreter = new PushGPInterpreter();
            var sw = new Stopwatch();

            sw.Start();
            for (var i = 0; i < 20000000; i++)
            {
                interpreter.Interprete(program);
                interpreter.Reset();
            }
            sw.Stop();

            Console.WriteLine(sw.Elapsed);
        }

        static void PerformanceTestCodeGenerator()
        {
            var sw = new Stopwatch();

            sw.Start();
            var expressions = CodeGenerator.RandomCode(10000000);
            sw.Stop();

            Console.WriteLine($"Count: {expressions.Count()}");
            Console.WriteLine(sw.Elapsed);
        }
    }
}