﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Algorithms.PushGP.Expressions;

namespace HeuristicLab.Algorithms.PushGP.Generators
{
    public static class CodeGenerator
    {
        public static IEnumerable<Expression> RandomCode(int maxPoints)
        {
            var actualPoints = StaticRandom.Next(1, maxPoints);

            return RandomCodeWithSize(actualPoints).ToArray();
        }

        private static IEnumerable<Expression> RandomCodeWithSize(int points)
        {
            if (points == 1)
            {
                OpCode opCode;
                Func<Expression> creator;

                do
                {
                    // TODO: If this is an "ephemeral random constant"???? then return a randomly - chosen value of the appropriate type;
                    opCode = (OpCode)StaticRandom.Next(OpCodeExtensions.Min, OpCodeExtensions.Max);
                }
                while (!ExpressionCreatorTable.TryGetCreator(opCode, out creator));

                return new[] { creator.Invoke() };
            }
            else
            {
                var sizesThisLevel = Decompose(points - 1, points - 1);

                return sizesThisLevel.SelectMany(size => RandomCodeWithSize(size));
            }
        }

        private static IEnumerable<int> Decompose(int number, int maxParts)
        {
            if (number == 1 || maxParts == 1)
            {
                return new[] { number };
            }
            else
            {
                var thisPart = StaticRandom.Next(1, number - 1);

                return new[] { thisPart }.Concat(Decompose(number - thisPart, maxParts - 1));
            }
        }
    }
}
