﻿using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.HeuristicLabAlgorithmsPushGP.Expressions
{
    [TestClass]
    public class IntegerExpressionTests : InterpreterTest
    {
        [TestMethod]
        public void TestAdd()
        {
            var code = "5 5 INTEGER.+";

            interpreter.Interprete(code);

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void TestAddOverflow()
        {
            var code = $"1 {long.MaxValue} INTEGER.+";

            interpreter.Interprete(code);

            Assert.AreEqual(long.MinValue, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void TestSubtract()
        {
            var code = "10 5 INTEGER.-";

            interpreter.Interprete(code);

            Assert.AreEqual(5, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void TestSubtractOverflow()
        {
            var code = $"{long.MinValue} 1 INTEGER.-";

            interpreter.Interprete(code);

            Assert.AreEqual(long.MaxValue, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }
    }
}
